package com.tbs.drag
{
	import com.tbs.utils.ObjectUtils;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	public class DragableItem extends Sprite
	{
		public static var avatarContainer:Sprite;
		public static var avatarBmp:Bitmap;
		public static var isDraging:Boolean;
		
		public static var itemSrc:DragableItem;
		
		public var dragable:Boolean;
		private var _mouseDownPoint:Point;
		private var _accept:Boolean;
		
		public function DragableItem()
		{
			
			init();
		}
		
		public static function init():void{
			if(!avatarContainer){
				avatarContainer = new Sprite();
				avatarContainer.mouseEnabled = false;
				avatarBmp = new Bitmap();
				avatarContainer.addChild(avatarBmp);
			}
			
			isDraging = false;
		}
		
		private function init():void{
			_accept = false;
			dragable = false;
			
			_mouseDownPoint = new Point();
			
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			addEventListener(MouseEvent.ROLL_OVER, onRollOver);
		}
		
		protected function onMouseDown(event:MouseEvent):void{
			stage.addEventListener(MouseEvent.MOUSE_UP, onStageMouseUp);
			if(dragable){
				_mouseDownPoint.x = event.localX;
				_mouseDownPoint.y = event.localY;
				beginDrag();
			}
		}
		
		protected function onMouseUp(event:MouseEvent):void{
			if(_accept){
				dragDrop(itemSrc);
			}
			
			endDrag();
			isDraging = false;
			if(itemSrc != this){
				removeEffect();
			}
		}
		
		protected function onStageMouseUp(event:MouseEvent):void{
			stage.removeEventListener(MouseEvent.MOUSE_UP, onStageMouseUp);
			if(isDraging){
				endDrag();
				isDraging = false;
				if(itemSrc != this){
					removeEffect();
				}
			}
		}
		
		protected function onRollOver(event:MouseEvent):void{
			addEventListener(MouseEvent.ROLL_OUT, onRollOut);
			
			if(isDraging && itemSrc != this){
				addEffect(_accept = dragEnter(itemSrc));
			}
		}
		
		protected function onRollOut(event:MouseEvent):void{
			removeEventListener(MouseEvent.ROLL_OUT, onRollOut);
			
			if(isDraging && itemSrc != this){
				removeEffect();
			}
		}
		
		/**
		 * 开始拖时
		 * 
		 */		
		protected function beginDrag():void{
			avatarBmp.bitmapData = dragAvatar;
			beforDrag();
			if(!stage.contains(avatarContainer)){
				stage.addChild(avatarContainer);
			}
			
			itemSrc = this;
			
			var sPoint:Point = this.localToGlobal(new Point(mouseX, mouseY));
			avatarContainer.x = sPoint.x - _mouseDownPoint.x;
			avatarContainer.y = sPoint.y - _mouseDownPoint.y;
			
			avatarContainer.startDrag();
			
			isDraging = true;
		}
		
		/**
		 * 结束拖放时
		 * 
		 */		
		protected function endDrag():void{
			if(stage.contains(avatarContainer)){
				stage.removeChild(avatarContainer);
			}
			
			avatarContainer.stopDrag();
		}
		
		/**
		 * 拖进时
		 * 
		 */		
		protected function dragEnter(itemSrc:DragableItem):Boolean{
			trace("dragEnter");
			
			return true;
		}
		
		/**
		 * 放下时
		 * 
		 */		
		protected function dragDrop(itemSrc:DragableItem):void{
			trace("dragDrop");
		}
		
		protected function beforDrag():void{
			avatarBmp.alpha = 0.8;
		}
		
		/**
		 * 拖进时的效果
		 * @param isAccept 是否接受
		 * 
		 */		
		protected function addEffect(isAccept:Boolean = true):void{
			trace("addEffect", isAccept ? "accept" : "forbid");
		}
		
		protected function removeEffect():void{
			trace("removeEffect");
		}
		
		protected function get dragAvatar():BitmapData{
			return ObjectUtils.cloneDisplayObject(this);
		}
	}
}